import { Component } from '@angular/core';
import { horizontalInOutReverse300ms } from '../../../../common.animation';
import { CommonNzModule } from '../../../../common.nz.module';
import { DataEmptyComponent } from '../../../../shared/data-empty/data-empty.component';
import { ApiService } from './../../../../services/api.service';
import { SettingService } from './../../../../services/setting.service';
import { CustomDrawerComponent } from './../../../../shared/custom-drawer/custom-drawer.component';
import { InspectionPlanDetailComponent } from './detail/detail.component';
import { cycleOptions, hazardPlanTypeOptions, planLevelOptions } from './inspection-plan.utils';
import { PlanCardComponent } from './plan-card/plan-card.component';
import { PlanFormComponent, PlanFormComponentProps } from './plan-form/plan-form.component';

export interface InspectionPlanDetailConfig {
  visible: boolean;
  data: Hazard.InspectionPlanDetail;
}

const notificationConfig = [
  { type: 1, text: '通知', color: '#165DFF', backgroundColor: '#E8F3FF' },
  { type: 2, text: '消息', color: '#0FC6C2', backgroundColor: '#E8FFFB' },
];

// const departments = Array.from({ length: 15 }, (_, i) => i + 1);
const getRandomSizeN = (max: number) => Math.floor(Math.random() * max);
const getRandomSize = (arr: Option<string | number>[]) => arr[Math.floor(Math.random() * arr.length)];

const getMockPlan = (index: number, name: string): Hazard.InspectionPlan => {
  return {
    id: index,
    type: getRandomSize(hazardPlanTypeOptions).value as string,
    title: `${name}隐患排查计划`,
    status: '',
    startDate: parseInt(name) + '-11-12T00:00:00',
    endDate: parseInt(name) + '-11-14T00:00:00',
    createdDate: parseInt(name) + '-11-10T00:00:00',
    departments: [getRandomSizeN(13) + 1],
    createUserId: '123',
    createUserName: '张三',
    createUserDept: 1,
    targets: [323, 346, 349, 16, 12, 8, 5, 4],
    level: getRandomSize(planLevelOptions).value as string,
    cycle: getRandomSize(cycleOptions).value as string,
    frequency: getRandomSizeN(4) || 1,
    frequencyTimes: getRandomSizeN(4) || 1,
    // time: [],
    content: '',
    isUnited: false,
  };
};

@Component({
  selector: 'app-inspection-plan',
  standalone: true,
  imports: [
    CommonNzModule,
    PlanCardComponent,
    InspectionPlanDetailComponent,
    CustomDrawerComponent,
    PlanFormComponent,
    DataEmptyComponent,
  ],
  templateUrl: './inspection-plan.component.html',
  styleUrl: './inspection-plan.component.less',
  animations: [horizontalInOutReverse300ms],
})
export class InspectionPlanComponent {
  detailConfig: InspectionPlanDetailConfig = {
    visible: false,
    data: {} as Hazard.InspectionPlanDetail,
  };
  formConfig: PlanFormComponentProps = {
    visible: false,
    data: undefined,
    isEdit: false,
  };
  semiAnnualOptions = [
    { label: '2023上半年度', value: 202301 },
    { label: '2023下半年度', value: 202306 },
    { label: '2024上半年度', value: 202401 },
    { label: '2024下半年度', value: 202406 },
    { label: '2025上半年度', value: 202501 },
  ];
  firstSection = [
    { label: '日常隐患排查计划', value: 3, key: 1 },
    { label: '专项隐患排查计划', value: 6, key: 2 },
    { label: '其他隐患排查计划', value: 9, key: 3 },
  ];
  secondSection = [
    { label: '计划站点排查覆盖', value: 65 },
    { label: '其他场所排查覆盖', value: 100 },
    { label: '计划人员排查占比', value: 95 },
  ];
  discovery = {
    significant: 1,
    a: 2,
    b: 3,
    c: 5,
  };
  notifications = [
    {
      type: 1,
      title:
        '当前产品试用期即将截止，在这段宝贵的试用时光里，您想必已经充分领略到了它的独特魅力与卓越性能。它犹如一位得力助手，在您的工作与生活中默默耕耘，助力您轻松跨越一道道难关，高效完成各项任务。无论是处理复杂的文档、分析海量的数据，还是畅享流畅的娱乐体验，它都表现得游刃有余',
      createDate: '2024-06-16T00:00:00',
    },
    {
      type: 1,
      title:
        '1月新系统升级计划通知,为了给您提供更优质、高效、稳定的服务体验，提升系统性能与功能，我们计划于 1 月进行系统升级。现将相关事宜详细通知如下：',
      createDate: '2024-10-16T00:00:00',
    },
    {
      type: 2,
      title:
        '新增内容已经通过审核，详情如下：此次审核涵盖了文本、图像、音频等多方面元素，确保了新增内容在质量与合规性上均达到了既定标准。文本部分，语言表达流畅自然，逻辑严谨清晰，无论是专业术语的运用还是普通语句的叙述，都经过了仔细的斟酌与校对，避免了歧义与错误信息的出现。',
      createDate: '2024-11-04T00:00:00',
    },
    {
      type: 1,
      title:
        '1月新系统升级计划通知为了给您提供更优质、高效、稳定的服务体验，提升系统性能与功能，我们计划于 1 月进行系统升级。现将相关事宜详细通知如下：',
      createDate: '2024-11-16T00:00:00',
    },
  ];
  currentPlanKey = 2;
  semiAnnualKey = 202406;

  // plans: Hazard.InspectionPlan[] = Array.from({ length: getRandomSizeN(30) }, (_, i) =>
  //   getMockPlan(i, this.getSemiAnnualLabel(this.semiAnnualKey))
  // );
  plans: Hazard.InspectionPlan[] = [];
  fetching = false;
  fetchingDetail = false;
  pageConfig: IPageConfig = {
    total: 0,
    page: 0,
    size: 12,
  };
  constructor(
    private api: ApiService,
    private setting: SettingService
  ) {
    setting.getUsersOfCurrentCompany();
  }

  ngOnInit() {
    this.fetchPlans();
  }

  fetchPlans() {
    this.fetching = true;
    this.api.hazard
      .getPlans({
        page: this.pageConfig.page,
        size: this.pageConfig.size,
      })
      .then(res => {
        this.plans = res.items;
        this.pageConfig.total = res.total;
      })
      .finally(() => {
        this.fetching = false;
      });
  }
  onCurrentPageChange(page: number) {
    this.pageConfig.page = page;
    this.fetchPlans();
  }
  handleRefreshList() {
    this.pageConfig.page = 0;
    this.fetchPlans();
  }
  getSemiAnnualLabel(key: number) {
    return this.semiAnnualOptions.find(s => s.value === key)?.label || '2025上半年度';
  }
  handlePlanTypeChange(type: number) {
    this.currentPlanKey = type;
  }
  handleSemiAnnualChange(direction: 'left' | 'right') {
    const index = this.currentSemiAnnualOptionIndex;
    if (direction === 'left') {
      if (!this.leftArrowVisible) return;
    }
    if (direction === 'right') {
      if (!this.rightArrowVisible) return;
    }
    this.semiAnnualKey = this.semiAnnualOptions[index + (direction === 'left' ? -1 : 1)]?.value || 0;
  }

  getNotificationTypeText(t: number) {
    return notificationConfig.find(n => n.type === t)?.text || '未知';
  }
  getNotificationTypeColor(t: number) {
    return notificationConfig.find(n => n.type === t)?.color || '';
  }
  getNotificationTypeBgColor(t: number) {
    return notificationConfig.find(n => n.type === t)?.backgroundColor || '';
  }
  handleCloseDetail() {
    this.detailConfig.visible = false;
  }
  handleViewDetail(data: Hazard.InspectionPlan) {
    this.handleFetchDetail(data.id);
    this.detailConfig = {
      ...this.detailConfig,
      visible: true,
      data: {
        ...data,
        plans: [],
      },
    };
  }

  handleAddPlan() {
    this.formConfig.visible = true;
    this.formConfig.isEdit = false;
  }
  handleEditPlan(data: Hazard.InspectionPlan) {
    this.formConfig.visible = true;
    this.formConfig.data = data;
    this.formConfig.isEdit = true;
  }
  closeFormDrawer() {
    this.formConfig.visible = false;
  }
  handleFetch() {
    this.plans = Array.from({ length: getRandomSizeN(30) }, (_, i) =>
      getMockPlan(i, this.getSemiAnnualLabel(this.semiAnnualKey))
    );
  }
  async handleFetchDetail(id: number) {
    this.fetchingDetail = true;
    await this.api.hazard
      .getPlanDetail(id)
      .then(res => {
        this.detailConfig = {
          ...this.detailConfig,
          visible: true,
          data: res,
        };
      })
      .finally(() => {
        this.fetchingDetail = false;
      });
  }
  get currentSemiAnnualOptionIndex() {
    return this.semiAnnualOptions.findIndex(se => se.value === this.semiAnnualKey);
  }
  get semiAnnualText() {
    return this.semiAnnualOptions.find(se => se.value === this.semiAnnualKey)?.label || '';
  }
  get leftArrowVisible() {
    const index = this.currentSemiAnnualOptionIndex;
    if (index === -1) return false;
    return !!this.semiAnnualOptions[index - 1];
  }
  get rightArrowVisible() {
    const index = this.currentSemiAnnualOptionIndex;
    if (index === -1) return false;
    return !!this.semiAnnualOptions[index + 1];
  }
}
